package j2d.filters.bufferedImageConvolution;

import gui.In;
import gui.layouts.VerticalLayout;
import gui.run.RunButton;
import gui.run.RunCheckBox;
import gui.run.RunComboBox;
import j2d.ImageProcessListener;
import j2d.ImageProcessorInterface;
import j2d.filters.fftConvolution.FFTR2ImageConvolverAM;

import javax.swing.*;
import java.awt.*;


/**
 * Created by Robert Distinti.
 * User: default
 * Date: Sep 19, 2005
 * Time: 1:53:41 AM
 */
public class ConvolutionControlFrame extends JInternalFrame {
    private ImageMatrixBean imb = ImageMatrixBean.restore();
    private ImageCheckBoxPanel mcbp;
    private ImageProcessListener tgt = null;
    private RunComboBox borderTypes=getBorderComboBox();
    private RunComboBox filterTypes = getFilterComboBox();


    // refresh is performed whenever size has changed
    private void refresh() {
        JInternalFrame cf = this;
        Container c = cf.getContentPane();
        c.remove(mcbp);
        mcbp = new ImageCheckBoxPanel(imb);
        c.add(mcbp, BorderLayout.CENTER);
        cf.pack();
        borderTypes.setSelectedIndex(imb.getBorderType());
        cf.setVisible(true);
    }

    public ConvolutionControlFrame(ImageProcessListener tg) {
        tgt = tg;
        setTitle("Select Image matrix");
        setup();
    }

    public ConvolutionControlFrame() {
        tgt = null;
        setTitle("Select Image matrix");
        setup();
    }
      private RunComboBox getFilterComboBox() {
        return new RunComboBox(new String[]{"convolution",
                                                   "fftr2",
                                                   "pfa"}) {
            public void run() {
                imb.setFilterType(getSelectedIndex());
                refresh();
            }
        };
    }
    private RunComboBox getBorderComboBox() {
        return new RunComboBox(new String[]{"Zero",
                                                   "Constant",
                                                   "Copy",
                                                   "Reflection",
                                                   "Wrap"}) {
            public void run() {
                imb.setBorderType((byte) getSelectedIndex());
                refresh();
            }
        };
    }

    private void setup() {
        JInternalFrame cf = this;
        Container c = cf.getContentPane();
        c.setLayout(new BorderLayout());

        mcbp = new ImageCheckBoxPanel(imb);
        c.add(mcbp, BorderLayout.CENTER);

        JPanel jp = new JPanel(new VerticalLayout());
        c.add(jp, BorderLayout.EAST);

        jp.add(new RunButton("ok") {
            public void run() {
                doFilter();

            }
        });
        jp.add(new RunButton("cancel") {
            public void run() {
                imb = ImageMatrixBean.restore();
                refresh();
            }
        });
        jp.add(new RunButton("resize") {
            public void run() {
                int size = In.getInt("input an odd number", 3, 33);
                mcbp.saveValues(); // save it to bean
                imb.setMatrixSize(size);
                refresh();
            }
        });
        jp.add(borderTypes);
        jp.add(filterTypes);
        setClosable(true);
        setMaximizable(true);
        setResizable(true);
        setVisible(true);
        refresh();
    }

    //todo implement hw6 here!<----------------------
    private void doFilter() {
        mcbp.saveValues(); // save it to bean
        imb.save(); // save bean to preferences
        ImageProcessorInterface op;
        if (imb.isRadix2())
            op = new FFTR2ImageConvolverAM(imb);
        else
            op = new ImageConvolver(imb);
        tgt.update(op);
    }
}
