package j2d.filters.bufferedImageConvolution;

import utils.CompactJava;

import java.awt.image.Kernel;
import java.io.*;
import java.util.prefs.Preferences;

/**
 * Robert Distinti
 * The image matrix bean -- modified from DocJava PreferencesBean
 */
public class ImageMatrixBean implements Serializable {
    private static final String key = "ece430.ImageMatrixBean";
    private String fileName = null;
    private byte borderType;
    private byte[][] data = new byte[5][5];
    private boolean isRadix2 = false;
    private int filterType = 0;
    public static final int CONVOLUTION =0;
    public static final int FFTR2 = 1;
    public static final int PFA = 2;


    /**
     * saves the properties to the Preferences of the userRoot
     */
    public void save() {
        try {
            Preferences p = Preferences.userRoot();
            ByteArrayOutputStream baos = new
                    ByteArrayOutputStream();
            ObjectOutputStream oos = new
                    ObjectOutputStream(baos);
            oos.writeObject(this);
            baos.close();
            p.putByteArray(key, baos.toByteArray());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public String toString() {
        return CompactJava.toXml(this);
    }

    /**
     * restores the properties from the preference in the user root.
     */
    public static ImageMatrixBean restore() {
        try {
            Preferences p = Preferences.userRoot();
            byte b [] = p.getByteArray(key, null);
            if (b == null)
                return new ImageMatrixBean();
            ByteArrayInputStream bais = new
                    ByteArrayInputStream(b);
            ObjectInputStream ois = new
                    ObjectInputStream(bais);
            Object o = ois.readObject();
            bais.close();
            return (ImageMatrixBean) o;
        } catch (IOException e) {
            //e.printStackTrace();
        } catch (ClassNotFoundException e) {
            // e.printStackTrace();
        }
        return new ImageMatrixBean();
    }

    public String getFileName() {
        return fileName;
    }

    public void setFileName(String fileName) {
        this.fileName = fileName;
    }

    public byte[][] getData() {
        return data;
    }

    public Kernel getKernel() {
        int matrixSize = data.length;
        float[] rtn = new float[matrixSize * matrixSize];
        int x, y, z;
        float sum = 0;
        z = 0;
        for (x = 0; x < matrixSize; x++) {
            for (y = 0; y < matrixSize; y++) {
                rtn[z] = data[x][y];
                sum += rtn[z];
                z++;
            }
        }
        // now normalize
        if (sum != 0) {
            for (x = 0; x < rtn.length; x++) {
                rtn[x] /= sum;
                System.out.println(rtn[x]);
            }
        }
        return new Kernel(matrixSize, matrixSize, rtn);
    }

    public void setData(byte[][] data) {
        this.data = data;

    }

    public int getMatrixSize() {
        return data.length;
    }

    public void setMatrixSize(int matrixSize) {
        int i, j, ofst, c;
        // first make sure it is odd
        matrixSize = matrixSize | 1;
        byte newData[][] = new byte[matrixSize][matrixSize];
        // first see if the size has changed
        int thisMatrixSize = data.length;
        if (thisMatrixSize == matrixSize) return;
        // see if we are expanding or contracting
        if ((thisMatrixSize > matrixSize)) {
            //contracting -- just take center elements
            ofst = (thisMatrixSize - matrixSize) / 2;
            c = matrixSize;
            for (i = 0; i < c; i++) {
                for (j = 0; j < c; j++) {
                    newData[i][j] = data[i + ofst][j + ofst];
                }
            }
        } else {
            // expanding -- keep it centered
            ofst = (matrixSize - thisMatrixSize) / 2;
            c = thisMatrixSize;
            for (i = 0; i < c; i++) {
                for (j = 0; j < c; j++) {
                    newData[i + ofst][j + ofst] = data[i][j];
                }
            }
        }
        thisMatrixSize = matrixSize;
        data = newData;
    }

    public byte getBorderType() {
        return borderType;
    }

    public void setBorderType(byte value) {
        borderType = value;
    }


    public boolean isRadix2() {
        return isRadix2;
    }

    public void setRadix2(boolean radix2) {
        isRadix2 = radix2;
    }

    public String getString() {
        byte b[][] = getData();
        StringBuffer sb = new StringBuffer();
        for (int x = 0; x < b.length; x++) {
            for (int y = 0; y < b[0].length; y++) {
                sb.append(b[x][y] + " ");
            }
            sb.append('\n');
        }
        return sb.toString();
    }

    public void setFilterType(int selectedIndex) {
        filterType = selectedIndex;
    }

    public int getFilterType() {
        return filterType;
    }
}
